import requests
from san.api_config import ApiConfig
from san.env_vars import SANBASE_GQL_HOST
from san.error import SanError


def execute_gql(gql_query_str):
    headers = {}
    if ApiConfig.api_key:
        headers = {'authorization':  "Apikey {}".format(ApiConfig.api_key)}

    response = requests.post(SANBASE_GQL_HOST, json={'query': gql_query_str}, headers=headers)

    if response.status_code == 200:
        if __exist_not_empty_result(response):
            return response.json()['data']
        else:
            raise SanError("Error running query. Status code: {}.\n {} \n errors: {}"
                .format(response.status_code, gql_query_str, response.json()['errors']))
    else:
        raise SanError("Error running query. Status code: {}.\n {} \n errors: {}"
            .format(response.status_code, gql_query_str, response.json()['errors']))

def __errors(response):
    return 'errors' in response.json().keys()
def __exist_not_empty_result(response):
    return len(list(filter(lambda x: x is not None, response.json()['data'].values()))) > 0



import datetime

def today():
    return datetime.date.today()

def two_days_ago():
    return (today() - datetime.timedelta(days=2)).isoformat()

def four_days_ago():
    return (today() - datetime.timedelta(days=4)).isoformat()

def month_ago():
    return (today() - datetime.timedelta(days=30)).isoformat()

class TestResponse:
    def __init__(self, **kwargs):
        self.status_code = kwargs['status_code']
        if 'errors' in kwargs['data']:
            self.data = kwargs['data']
        else:
            self.data = {'data': kwargs['data']}

    def json(self):
        return self.data

    def status_code(self):
        return self.status_code



from san import Batch
from unittest.mock import Mock, patch
from san.pandas_utils import convert_to_datetime_idx_df
import pandas.testing as pdt
from san.tests.utils import TestResponse

@patch('san.graphql.requests.post')
def test_batch(mock):
    expected = {
        'query_0':
            [{'datetime': '2018-06-01T00:00:00Z', 'activeAddresses': 2},
             {'datetime': '2018-06-02T00:00:00Z', 'activeAddresses': 4},
             {'datetime': '2018-06-03T00:00:00Z', 'activeAddresses': 6},
             {'datetime': '2018-06-04T00:00:00Z', 'activeAddresses': 6},
             {'datetime': '2018-06-05T00:00:00Z', 'activeAddresses': 14}],
        'query_1':
            [{'datetime': '2018-06-06T00:00:00Z', 'activeAddresses': 2},
             {'datetime': '2018-06-07T00:00:00Z', 'activeAddresses': 10},
             {'datetime': '2018-06-08T00:00:00Z', 'activeAddresses': 21},
             {'datetime': '2018-06-09T00:00:00Z', 'activeAddresses': 6},
             {'datetime': '2018-06-10T00:00:00Z', 'activeAddresses': 5}]
    }
    mock.return_value = TestResponse(status_code=200, data=expected)

    batch = Batch()
    batch.get(
        "daily_active_addresses/santiment",
        from_date="2018-06-01",
        to_date="2018-06-05",
        interval="1d"
    )
    batch.get(
        "daily_active_addresses/santiment",
        from_date="2018-06-06",
        to_date="2018-06-10",
        interval="1d"
    )
    [res1, res2] = batch.execute()

    df1 = convert_to_datetime_idx_df(expected['query_0'])
    df2 = convert_to_datetime_idx_df(expected['query_1'])

    pdt.assert_frame_equal(res1, df1, check_dtype=False)
    pdt.assert_frame_equal(res2, df2, check_dtype=False)



import iso8601
import datetime
from san.pandas_utils import merge
from san.batch import Batch
from san.error import SanError


DEFAULT_INTERVAL = '1d'
DEFAULT_SOCIAL_VOLUME_TYPE = 'PROFESSIONAL_TRADERS_CHAT_OVERVIEW'
DEFAULT_SOURCE = 'TELEGRAM'
DEFAULT_SEARCH_TEXT = ''

QUERY_MAPPING = {
    'daily_active_addresses': {
        'query': 'dailyActiveAddresses',
        'return_fields': ['datetime', 'activeAddresses']
    },
    'burn_rate': { # to be removed
        'query': 'burnRate',
        'return_fields': ['datetime', 'burnRate']
    },
    'token_age_consumed': {
        'query': 'tokenAgeConsumed',
        'return_fields': ['datetime', 'tokenAgeConsumed']
    },
    'average_token_age_consumed_in_days': {
        'query': 'averageTokenAgeConsumedInDays',
        'return_fields': ['datetime', 'tokenAge']
    },
    'transaction_volume': {
        'query': 'transactionVolume',
        'return_fields': ['datetime', 'transactionVolume']
    },
    'github_activity': {
        'query': 'githubActivity',
        'return_fields': ['datetime', 'activity']
    },
    'dev_activity': {
        'query': 'devActivity',
        'return_fields': ['datetime', 'activity']
    },
    'network_growth': {
        'query': 'networkGrowth',
        'return_fields': ['datetime', 'newAddresses']
    },
    'prices': {
        'query': 'historyPrice',
        'return_fields': ['datetime', 'priceUsd', 'priceBtc', 'marketcap', 'volume']
    },
    'ohlc': {
        'query': 'ohlc',
        'return_fields': ['datetime', 'openPriceUsd', 'closePriceUsd', 'highPriceUsd', 'lowPriceUsd']
    },
    'exchange_funds_flow': {
        'query': 'exchangeFundsFlow',
        'return_fields': ['datetime', 'inOutDifference']
    },
    'token_velocity': {
        'query': 'tokenVelocity',
        'return_fields': ['datetime', 'tokenVelocity']
    },
    'token_circulation': {
        'query': 'tokenCirculation',
        'return_fields': ['datetime', 'tokenCirculation']
    },
    'realized_value': {
        'query': 'realizedValue',
        'return_fields': ['datetime', 'nonExchangeRealizedValue', 'realizedValue']
    },
    'mvrv_ratio': {
        'query': 'mvrvRatio',
        'return_fields': ['datetime', 'ratio']
    },
    'nvt_ratio': {
        'query': 'nvtRatio',
        'return_fields': ['datetime', 'nvtRatioCirculation', 'nvtRatioTxVolume']
    },
    'daily_active_deposits': {
        'query': 'dailyActiveDeposits',
        'return_fields': ['datetime', 'activeDeposits']
    },
    'share_of_deposits':{
        'query': 'shareOfDeposits',
        'return_fields': ['activeAddresses','activeDeposits','datetime', 'shareOfDeposits']
    }
}


def daily_active_addresses(idx, slug, **kwargs):
    query_str = _create_query_str(
        'daily_active_addresses', idx, slug, **kwargs)

    return query_str

# to be removed
def burn_rate(idx, slug, **kwargs):
    query_str = _create_query_str('burn_rate', idx, slug, **kwargs)

    return query_str

def transaction_volume(idx, slug, **kwargs):
    query_str = _create_query_str('transaction_volume', idx, slug, **kwargs)

    return query_str

def token_age_consumed(idx, slug, **kwargs):
    query_str = _create_query_str('token_age_consumed', idx, slug, **kwargs)

    return query_str

def average_token_age_consumed_in_days(idx, slug, **kwargs):
    query_str = _create_query_str('average_token_age_consumed_in_days', idx, slug, **kwargs)

    return query_str


def github_activity(idx, slug, **kwargs):
    query_str = _create_query_str('github_activity', idx, slug, **kwargs)

    return query_str

def dev_activity(idx, slug, **kwargs):
    query_str = _create_query_str('dev_activity', idx, slug, **kwargs)

    return query_str

def network_growth(idx, slug, **kwargs):
    query_str = _create_query_str('network_growth', idx, slug, **kwargs)

    return query_str

def prices(idx, slug, **kwargs):
    query_str = _create_query_str('prices', idx, slug, **kwargs)

    return query_str

def token_velocity(idx, slug, **kwargs):
    query_str = _create_query_str('token_velocity', idx, slug, **kwargs)

    return query_str

def token_circulation(idx, slug, **kwargs):
    query_str = _create_query_str('token_circulation', idx, slug, **kwargs)

    return query_str

def realized_value(idx, slug, **kwargs):
    query_str = _create_query_str('realized_value', idx, slug, **kwargs)

    return query_str

def mvrv_ratio(idx, slug, **kwargs):
    query_str = _create_query_str('mvrv_ratio', idx, slug, **kwargs)

    return query_str

def nvt_ratio(idx, slug, **kwargs):
    query_str = _create_query_str('nvt_ratio', idx, slug, **kwargs)

    return query_str

def daily_active_deposits(idx, slug, **kwargs):
    query_str = _create_query_str('daily_active_deposits', idx, slug, **kwargs)

    return query_str

def share_of_deposits(idx, slug, **kwargs):
    query_str = _create_query_str('share_of_deposits', idx, slug, **kwargs)

    return query_str

def ohlc(idx, slug, **kwargs):
    query_str = _create_query_str('ohlc', idx, slug, **kwargs)

    return query_str


def ohlcv(idx, slug, **kwargs):
    return_fields = ['openPriceUsd', 'closePriceUsd', 'highPriceUsd', 'lowPriceUsd', 'volume', 'marketcap']

    batch = Batch()
    batch.get(
        "prices/{slug}".format(slug=slug),
        **kwargs
    )
    batch.get(
        "ohlc/{slug}".format(slug=slug),
        **kwargs
    )
    [price_df, ohlc_df] = batch.execute()
    merged = merge(price_df, ohlc_df)
    return merged[return_fields]

def projects(idx, slug, **kwargs):
    if (slug == "erc20"):
        return erc20_projects(idx, **kwargs)
    elif (slug == "all"):
        return all_projects(idx, **kwargs)

    raise SanError("Unknown project group: {}".format(slug))

def all_projects(idx, **kwargs):
    query_str = """
    query_{idx}: allProjects
    {{
        name,
        slug,
        ticker,
        totalSupply,
        marketSegment
    }}
    """.format(idx=idx)

    return query_str

def erc20_projects(idx, **kwargs):
    query_str = """
    query_{idx}: allErc20Projects
    {{
        name,
        slug,
        ticker,
        totalSupply,
        marketSegment
    }}
    """.format(idx=idx)

    return query_str


def exchange_funds_flow(idx, slug, **kwargs):
    query_str = _create_query_str('exchange_funds_flow', idx, slug, **kwargs)

    return query_str


def social_volume_projects(idx, _slug, **kwargs):
    query_str = """
    query_{idx}: socialVolumeProjects
    """.format(idx=idx)

    return query_str


def social_volume(idx, slug, **kwargs):
    kwargs = _transform_query_args(**kwargs)

    query_str = """
    query_{idx}: socialVolume (
        slug: \"{slug}\",
        from: \"{from_date}\",
        to: \"{to_date}\",
        interval: \"{interval}\",
        socialVolumeType: {social_volume_type}
    ){{
        mentionsCount,
        datetime
    }}
    """.format(
        idx=idx,
        slug=slug,
        **kwargs
    )

    return query_str


def topic_search(idx, field, **kwargs):
    kwargs = _transform_query_args(**kwargs)
    return_fields = {
        'messages': """
        messages {
            datetime
            text
        }
        """,
        'chart_data': """
        chartData {
            mentionsCount
            datetime
        }
        """
    }

    query_str = """
    query_{idx}: topicSearch (
        source: {source},
        searchText: \"{search_text}\",
        from: \"{from_date}\",
        to: \"{to_date}\",
        interval: \"{interval}\"
    ){{
        {return_fields}
    }}
    """.format(
        idx=idx,
        return_fields=return_fields[field],
        **kwargs
    )

    return query_str


def _create_query_str(query, idx, slug, **kwargs):
    kwargs = _transform_query_args(**kwargs)

    query_str = """
    query_{idx}: {query}(
        slug: \"{slug}\",
        from: \"{from_date}\",
        to: \"{to_date}\",
        interval: \"{interval}\"
    ){{
        {return_fields}
    }}
    """.format(
        query=QUERY_MAPPING[query]['query'],
        idx=idx,
        slug=slug,
        return_fields=_format_return_fields(
            QUERY_MAPPING[query]['return_fields']),
        **kwargs
    )
    
    return query_str


def _transform_query_args(**kwargs):
    kwargs['from_date'] = kwargs['from_date'] if 'from_date' in kwargs else _default_from_date()
    kwargs['to_date'] = kwargs['to_date'] if 'to_date' in kwargs else _default_to_date()
    kwargs['interval'] = kwargs['interval'] if 'interval' in kwargs else DEFAULT_INTERVAL
    kwargs['social_volume_type'] = kwargs['social_volume_type'] if 'social_volume_type' in kwargs else DEFAULT_SOCIAL_VOLUME_TYPE
    kwargs['source'] = kwargs['source'] if 'source' in kwargs else DEFAULT_SOURCE
    kwargs['search_text'] = kwargs['search_text'] if 'search_text' in kwargs else DEFAULT_SEARCH_TEXT

    kwargs['from_date'] = _format_from_date(kwargs['from_date'])
    kwargs['to_date'] = _format_to_date(kwargs['to_date'])

    return kwargs


def _default_to_date():
    return datetime.datetime.now()


def _default_from_date():
    return datetime.datetime.now() - datetime.timedelta(days=365)


def _format_return_fields(return_fields):
    return ",\n".join(return_fields)


def _format_from_date(datetime_obj_or_str):
    if isinstance(datetime_obj_or_str, datetime.datetime):
        datetime_obj_or_str = datetime_obj_or_str.isoformat()

    return iso8601.parse_date(datetime_obj_or_str).isoformat()

def _format_to_date(datetime_obj_or_str):
    if isinstance(datetime_obj_or_str, datetime.datetime):
        datetime_obj_or_str = datetime_obj_or_str.isoformat()

    dt = iso8601.parse_date(datetime_obj_or_str) + datetime.timedelta(hours=23, minutes=59, seconds=59)
    return dt.isoformat()




import san.sanbase_graphql
from san.error import SanError

def get_gql_query(idx, dataset, **kwargs):
    query, slug = parse_dataset(dataset)
    return getattr(
        san.sanbase_graphql,
        query,
        lambda *args, **kwargs: not_found(query)
    )(idx, slug, **kwargs)


def parse_dataset(dataset):
    left, _separator, right = dataset.partition("/")
    return [left, right]


def not_found(query):
    raise SanError(query + ' not found')



from san.query import get_gql_query
from san.graphql import execute_gql
from san.pandas_utils import convert_to_datetime_idx_df

class Batch:

    def __init__(self):
        self.queries = []

    def get(self, dataset, **kwargs):
        self.queries.append([dataset, kwargs])

    def execute(self):
        result = []
        batched_queries = []

        for idx, q in enumerate(self.queries):
            batched_queries.append(get_gql_query(idx, q[0], **q[1]))

        gql_string = self.__batch_gql_queries(batched_queries)
        res = execute_gql(gql_string)

        idxs = sorted([int(k.split('_')[1]) for k in res.keys()])
        for idx in idxs:
            key = "query_{}".format(idx)
            df = convert_to_datetime_idx_df(res[key])
            result.append(df)

        return result

    def __batch_gql_queries(self, batched_queries):
        gql_string = "{\n"
        for q in batched_queries:
            gql_string = gql_string + q + "\n"

        gql_string = gql_string + "}"
        return gql_string



import san
from san.error import SanError
from nose.tools import assert_raises
from unittest.mock import Mock, patch
from san.pandas_utils import convert_to_datetime_idx_df
import pandas.testing as pdt
from san.graphql import execute_gql
from san.tests.utils import TestResponse

@patch('san.graphql.requests.post')
def test_get(mock):
    expected = {
        'query_0':
            [{'datetime': '2018-06-01T00:00:00Z', 'activeAddresses': 2},
             {'datetime': '2018-06-02T00:00:00Z', 'activeAddresses': 4},
             {'datetime': '2018-06-03T00:00:00Z', 'activeAddresses': 6},
             {'datetime': '2018-06-04T00:00:00Z', 'activeAddresses': 6},
             {'datetime': '2018-06-05T00:00:00Z', 'activeAddresses': 14}]
    }
    mock.return_value = TestResponse(status_code=200, data=expected)

    res = san.get(
        "daily_active_addresses/santiment",
        from_date="2018-06-01",
        to_date="2018-06-05",
        interval="1d"
    )
    df = convert_to_datetime_idx_df(expected['query_0'])
    pdt.assert_frame_equal(res, df, check_dtype=False)


@patch('san.graphql.requests.post')
def test_prices(mock):
    expected = {
        'query_0':
            [
                {
                    'priceUsd': '1.234634930555555',
                    'priceBtc': '0.0001649780416666666',
                    'datetime': '2018-06-01T00:00:00Z'
                },
                {
                    'priceUsd': '1.2551352777777771',
                    'priceBtc': '0.00016521851041666669',
                    'datetime': '2018-06-02T00:00:00Z'
                },
                {
                    'priceUsd': '1.251881943462897',
                    'priceBtc': '0.000162902558303887',
                    'datetime': '2018-06-03T00:00:00Z'
                },
                {
                    'priceUsd': '1.2135782638888888',
                    'priceBtc': '0.0001600935277777778',
                    'datetime': '2018-06-04T00:00:00Z'
                }
            ]
    }
    mock.return_value = TestResponse(status_code=200, data=expected)

    res = san.get(
        "prices/santiment_usd",
        from_date="2018-06-01",
        to_date="2018-06-05",
        interval="1d"
    )

    df = convert_to_datetime_idx_df(expected['query_0'])
    pdt.assert_frame_equal(res, df, check_dtype=False)

@patch('san.graphql.requests.post')
def test_error_response(mock):
    mock.return_value = TestResponse(status_code=500, data={'errors': {'detail': 'Internal server error'}})
    with assert_raises(SanError):
        san.get(
            "prices/santiment_usd",
            from_date="2018-06-01",
            to_date="2018-06-05",
            interval="1d"
        )

def test_invalid_request():
    with assert_raises(ValueError):
        san.get("invalid_request")


def test_invalid_method():
    with assert_raises(SanError):
        san.get("invalid_method/slug")



from .get import get
from .batch import Batch
from .api_config import ApiConfig
import pkg_resources
import requests
import json
from warnings import warn

PROJECT = 'sanpy'

def get_latest():
    url = 'https://pypi.python.org/pypi/%s/json' % (PROJECT)
    try:
      response = requests.get(url).text
      return json.loads(response)['info']['version']
    except requests.exceptions.RequestException as e:
      return pkg_resources.get_distribution(PROJECT).version

def warn_if_outdated(): 
    current_version = pkg_resources.get_distribution(PROJECT).version
    latest_version = get_latest()

    if current_version != latest_version:
      warning = 'The package %s is out of date. Your version is %s, the latest is %s.' % (PROJECT, current_version, latest_version)
      warn(warning)
      pass

warn_if_outdated()





import san
from san import Batch
from san import sanbase_graphql
from san.tests.utils import two_days_ago, four_days_ago, month_ago

import datetime
from nose.plugins.attrib import attr

params = {
    "project_slug":  "santiment",
    "from_date": month_ago(),
    "to_date": two_days_ago(),
    "interval": "1d"
}

@attr('integration')
def test_batched_queries_equal_format():
    queries = sanbase_graphql.QUERY_MAPPING.keys()

    batch = Batch()
    for query in queries:
        batch.get(
            "{}/{}".format(query, params["project_slug"]),
            from_date=params["from_date"],
            to_date=params["to_date"],
            interval=params["interval"]
        )

    result = batch.execute()

    for df in result:
        assert len(df.index) >= 1

@attr('integration')
def test_batched_queries_different_format():
    batch = Batch()

    batch.get("projects/all")
    batch.get("social_volume_projects")
    batch.get(
        "{}/{}".format('social_volume', 'ethereum'),
        from_date=month_ago(),
        to_date=params["to_date"],
        interval=params["interval"],
        social_volume_type="PROFESSIONAL_TRADERS_CHAT_OVERVIEW"
    )
    batch.get(
        "topic_search/messages",
        source="TELEGRAM",
        search_text="btc",
        from_date=month_ago(),
        to_date=params["to_date"],
        interval=params["interval"]
    )

    batch.get(
        "topic_search/chart_data",
        source="TELEGRAM",
        search_text="btc",
        from_date=month_ago(),
        to_date=params["to_date"],
        interval=params["interval"]
    )

    result = batch.execute()
    for df in result:
        assert len(df.index) >= 1

@attr('integration')
def test_erc20_projects():
    result = san.get("projects/erc20")

    assert len(result) > 0
    assert len(result[result.slug == "bitcoin"]) == 0

@attr('integration')
def test_ohlcv():
    ohlcv_df = san.get(
        "{}/{}".format('ohlcv', params["project_slug"]),
        from_date=params["from_date"],
        to_date=params["to_date"],
        interval=params["interval"]
    )

    assert len(ohlcv_df.index) >= 1



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="sanpy",
    version="0.5.2",
    author="Santiment",
    author_email="admin@santiment.net",
    description="Package for Santiment API access with python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/santiment/sanpy",
    test_suite='nose.collector',
    tests_require=['nose'],
    packages=setuptools.find_packages(),
    install_requires=[
        'pandas',
        'numpy',
        'requests',
        'iso8601',
        'setuptools'
    ]
)



import san.sanbase_graphql
from san.graphql import execute_gql
from san.pandas_utils import convert_to_datetime_idx_df
from san.query import get_gql_query, parse_dataset

CUSTOM_QUERIES = {
    'ohlcv': 'get_ohlcv',
}

def get(dataset, **kwargs):
    query, slug = parse_dataset(dataset)
    if query in CUSTOM_QUERIES:
        return getattr(san.sanbase_graphql, query)(0, slug, **kwargs)

    gql_query = get_gql_query(0, dataset, **kwargs)
    gql_query = "{\n" + gql_query + "\n}"
    res = execute_gql(gql_query)

    return convert_to_datetime_idx_df(res["query_0"])



import pandas as pd


def convert_to_datetime_idx_df(data):
    df = pd.DataFrame(data)

    if 'datetime' in df.columns:
        df['datetime'] = pd.to_datetime(df['datetime'], utc=True)
        df.set_index('datetime', inplace=True)

    return df


def merge(df1, df2):
    return pd.concat([df1, df2], axis=1)
